from elevenlabs.endpoint import Endpoint
from elevenlabs.voices.voice import Voice
from elevenlabs.voices.voices_iter import VoicesIter

import time

class Voices(Endpoint):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._voices = {}

    def __iter__(self):
        """ Return a unique Iterator for iterating through all voices """
        return VoicesIter(self)

    def __getitem__(self, voice_identifier):
        """ Returns a Voice object by voice name, UUID, or index """

        # If int, return a voice by index on list
        if type(voice_identifier) == int:
            return self.list[voice_identifier]

        # Otherwise, iter through all voices and find first matching voice
        # by name or UUID.
        for voice in self.list:
            # Match by UUID
            if voice.id == voice_identifier:
                return voice

            # Match by name - non-case sensitive
            if voice.name.lower() == voice_identifier.lower():
                return voice

        raise KeyError(voice_identifier)

    def _refresh_voice_list(self):
        r = self._request("GET", "voices/")
        payload = r.json()

        for voice_obj in payload["voices"]:
            voice_id = voice_obj["voice_id"]

            if voice_id not in self._voices:
                voice = Voice(self, voice_obj)

                self._voices[voice_id] = voice

    @property
    def list(self):
        self._throttle(
            "voicelist",
            60,
            self._refresh_voice_list
        )

        return [self._voices[voice_id] for voice_id in self._voices]



from tempfile import SpooledTemporaryFile

import mimetypes

class AudioFile:
    def __init__(self, request):
        self._request = request

        self._file = SpooledTemporaryFile()
        self._file.write(request.content)

    def save(self, filename):
        """ Saves the resulting audio file to *filename*, with the
        appropriate file extension automatically appended to the end of
        the path. """

        content_type = self._request.headers["content-type"]
        extension = mimetypes.guess_extension(content_type)

        filename = "%s%s" % (filename, extension)

        self._file.seek(0)

        with open(filename, "wb") as f:
            f.write(
                self._file.read()
            )



from elevenlabs.endpoint import Endpoint

class TTS(Endpoint):
    """ I was winging this whole library, so I thought to match the API
        documentation one-to-one, but I realized I could do better. I could make
        it far more Pythonic. So I probably won't be using this now. """



from elevenlabs.voices.audiofile import AudioFile

import requests

class Voice:
    def __init__(self, voices, obj):
        self._voices = voices
        self._obj = obj

        self._request = self._voices._request

    @property
    def id(self):
        return self._obj["voice_id"]

    @property
    def name(self):
        return self._obj["name"]

    @property
    def category(self):
        return self._obj["category"]

    @property
    def preview_url(self):
        return self._obj["preview_url"]

    @property
    def preview(self):
        request = requests.get(self.preview_url)

        return AudioFile(request)

    @property
    def settings(self):
        request = self._request(
            "GET",
            "voices/%s/settings" % self.id
        )

        return request.json()

    def generate(self, text, voice_settings=None):
        if not voice_settings:
            voice_settings = self.settings

        request = self._request(
            "POST",
            "text-to-speech/%s" % self.id,
            {
                "text": text,
                "voice_settings": voice_settings
            }
        )

        return AudioFile(request)



import requests
import time
import os

from elevenlabs.exceptions import *

from elevenlabs.voices import Voices

class ElevenLabs:
    def __init__(self, api_key):
        self.api_key = api_key
        self.endpoint = "https://api.elevenlabs.io/v1/"

        self.voices = Voices(self)

        self._throttles = {}

    def _throttle(self, id, freq, callback):
        if id not in self._throttles:
            self._throttles[id] = time.time()

            callback()
            return

        duration = time.time() - self._throttles[id]

        if duration < freq:
            callback()

    def _request(self, method, path, body=None):
        url = self.endpoint + path

        request = requests.request(
            method,
            url,
            json=body,
            headers={
                "xi-api-key": self.api_key
            }
        )

        if request.status_code != 200:
            # TODO: throw a proper exception here, add more specific handling

            print(request.content)
            
            raise Exception("%s" % (
                request.status_code
            ))

        return request



from setuptools import setup, find_packages

# Not gonna lie; I used ChatGPT to generate this setup.py. Nifty.

setup(
    name='elevenlabs',
    version='0.1',
    description='A Python library for integrating with ElevenLabs.io\'s API',
    long_description=open("README.rst", "r").read(),
    author='Ben Baptist',
    author_email='me@benbaptist.com',
    url='https://github.com/benbaptist/elevenlabs',
    packages=find_packages(),
    install_requires=[
        'requests',
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



class VoicesIter:
    def __init__(self, voices):
        self.list = voices.list

        self.i = 0

    def __iter__(self):
        return self

    def __next__(self):
        i = self.i
        self.i += 1

        try:
            return self.list[i]
        except IndexError:
            raise StopIteration
